#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Convenience wrapper for running Onionbalance directly from source tree."""

import onionbalance.manager

if __name__ == '__main__':
    onionbalance.manager.main()
