#! /usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Convenience wrapper for running onionbalance config generator directly from source tree.
"""

import onionbalance.config_generator.config_generator

if __name__ == '__main__':
    onionbalance.config_generator.config_generator.main()

